@@ -21,7 +21,7 @@ def shortcut(request, content_type_id, object_id):
         obj = content_type.get_object_for_this_type(pk=object_id)
     except (ObjectDoesNotExist, ValueError):
         raise Http404(
-            _("Content type %(ct_id)s object %(obj_id)s doesn't exist") %
+            _('Content type %(ct_id)s object %(obj_id)s doesn’t exist') %
             {'ct_id': content_type_id, 'obj_id': object_id}
         )
 
@@ -29,7 +29,7 @@ def shortcut(request, content_type_id, object_id):
         get_absolute_url = obj.get_absolute_url
     except AttributeError:
         raise Http404(
-            _("%(ct_name)s objects don't have a get_absolute_url() method") %
+            _('%(ct_name)s objects don’t have a get_absolute_url() method') %
             {'ct_name': content_type.name}
         )
     absurl = get_absolute_url()
