@@ -1,6 +1,6 @@
 from django.contrib.sites.models import Site
 from django.db import models
-from django.urls import get_script_prefix
+from django.urls import NoReverseMatch, get_script_prefix, reverse
 from django.utils.encoding import iri_to_uri
 from django.utils.translation import gettext_lazy as _
 
@@ -15,8 +15,8 @@ class FlatPage(models.Model):
         max_length=70,
         blank=True,
         help_text=_(
-            "Example: 'flatpages/contact_page.html'. If this isn't provided, "
-            "the system will use 'flatpages/default.html'."
+            'Example: “flatpages/contact_page.html”. If this isn’t provided, '
+            'the system will use “flatpages/default.html”.'
         ),
     )
     registration_required = models.BooleanField(
@@ -30,11 +30,18 @@ class FlatPage(models.Model):
         db_table = 'django_flatpage'
         verbose_name = _('flat page')
         verbose_name_plural = _('flat pages')
-        ordering = ('url',)
+        ordering = ['url']
 
     def __str__(self):
         return "%s -- %s" % (self.url, self.title)
 
     def get_absolute_url(self):
+        from .views import flatpage
+
+        for url in (self.url.lstrip('/'), self.url):
+            try:
+                return reverse(flatpage, kwargs={'url': url})
+            except NoReverseMatch:
+                pass
         # Handle script prefix manually because we bypass reverse()
         return iri_to_uri(get_script_prefix().rstrip('/') + self.url)
