@@ -1,3 +1,5 @@
+from decimal import Decimal
+
 from django.conf import settings
 from django.template import Library, Node, TemplateSyntaxError, Variable
 from django.template.base import TokenType, render_value_in_context
@@ -98,7 +100,8 @@ class TranslateNode(Node):
 class BlockTranslateNode(Node):
 
     def __init__(self, extra_context, singular, plural=None, countervar=None,
-                 counter=None, message_context=None, trimmed=False, asvar=None):
+                 counter=None, message_context=None, trimmed=False, asvar=None,
+                 tag_name='blocktranslate'):
         self.extra_context = extra_context
         self.singular = singular
         self.plural = plural
@@ -107,6 +110,7 @@ class BlockTranslateNode(Node):
         self.message_context = message_context
         self.trimmed = trimmed
         self.asvar = asvar
+        self.tag_name = tag_name
 
     def render_token_list(self, tokens):
         result = []
@@ -133,6 +137,11 @@ class BlockTranslateNode(Node):
         singular, vars = self.render_token_list(self.singular)
         if self.plural and self.countervar and self.counter:
             count = self.counter.resolve(context)
+            if not isinstance(count, (Decimal, float, int)):
+                raise TemplateSyntaxError(
+                    "%r argument to %r tag must be a number."
+                    % (self.countervar, self.tag_name)
+                )
             context[self.countervar] = count
             plural, plural_vars = self.render_token_list(self.plural)
             if message_context:
@@ -163,8 +172,8 @@ class BlockTranslateNode(Node):
             if nested:
                 # Either string is malformed, or it's a bug
                 raise TemplateSyntaxError(
-                    "'blocktrans' is unable to format string returned by gettext: %r using %r"
-                    % (result, data)
+                    '%r is unable to format string returned by gettext: %r '
+                    'using %r' % (self.tag_name, result, data)
                 )
             with translation.override(None):
                 result = self.render(context, nested=True)
@@ -313,6 +322,7 @@ def do_get_current_language_bidi(parser, token):
     return GetCurrentLanguageBidiNode(args[2])
 
 
+@register.tag("translate")
 @register.tag("trans")
 def do_translate(parser, token):
     """
@@ -321,7 +331,7 @@ def do_translate(parser, token):
 
     Usage::
 
-        {% trans "this is a test" %}
+        {% translate "this is a test" %}
 
     This marks the string for translation so it will be pulled out by
     makemessages into the .po files and runs the string through the translation
@@ -329,7 +339,7 @@ def do_translate(parser, token):
 
     There is a second form::
 
-        {% trans "this is a test" noop %}
+        {% translate "this is a test" noop %}
 
     This marks the string for translation, but returns the string unchanged.
     Use it when you need to store values into forms that should be translated
@@ -338,19 +348,19 @@ def do_translate(parser, token):
     You can use variables instead of constant strings
     to translate stuff you marked somewhere else::
 
-        {% trans variable %}
+        {% translate variable %}
 
     This tries to translate the contents of the variable ``variable``. Make
     sure that the string in there is something that is in the .po file.
 
     It is possible to store the translated string into a variable::
 
-        {% trans "this is a test" as var %}
+        {% translate "this is a test" as var %}
         {{ var }}
 
     Contextual translations are also supported::
 
-        {% trans "this is a test" context "greeting" %}
+        {% translate "this is a test" context "greeting" %}
 
     This is equivalent to calling pgettext instead of (u)gettext.
     """
@@ -406,6 +416,7 @@ def do_translate(parser, token):
     return TranslateNode(message_string, noop, asvar, message_context)
 
 
+@register.tag("blocktranslate")
 @register.tag("blocktrans")
 def do_block_translate(parser, token):
     """
@@ -413,37 +424,37 @@ def do_block_translate(parser, token):
 
     Usage::
 
-        {% blocktrans with bar=foo|filter boo=baz|filter %}
+        {% blocktranslate with bar=foo|filter boo=baz|filter %}
         This is {{ bar }} and {{ boo }}.
-        {% endblocktrans %}
+        {% endblocktranslate %}
 
     Additionally, this supports pluralization::
 
-        {% blocktrans count count=var|length %}
+        {% blocktranslate count count=var|length %}
         There is {{ count }} object.
         {% plural %}
         There are {{ count }} objects.
-        {% endblocktrans %}
+        {% endblocktranslate %}
 
     This is much like ngettext, only in template syntax.
 
     The "var as value" legacy format is still supported::
 
-        {% blocktrans with foo|filter as bar and baz|filter as boo %}
-        {% blocktrans count var|length as count %}
+        {% blocktranslate with foo|filter as bar and baz|filter as boo %}
+        {% blocktranslate count var|length as count %}
 
     The translated string can be stored in a variable using `asvar`::
 
-        {% blocktrans with bar=foo|filter boo=baz|filter asvar var %}
+        {% blocktranslate with bar=foo|filter boo=baz|filter asvar var %}
         This is {{ bar }} and {{ boo }}.
-        {% endblocktrans %}
+        {% endblocktranslate %}
         {{ var }}
 
     Contextual translations are supported::
 
-        {% blocktrans with bar=foo|filter context "greeting" %}
+        {% blocktranslate with bar=foo|filter context "greeting" %}
             This is {{ bar }}.
-        {% endblocktrans %}
+        {% endblocktranslate %}
 
     This is equivalent to calling pgettext/npgettext instead of
     (u)gettext/(u)ngettext.
@@ -513,19 +524,20 @@ def do_block_translate(parser, token):
             break
     if countervar and counter:
         if token.contents.strip() != 'plural':
-            raise TemplateSyntaxError("'blocktrans' doesn't allow other block tags inside it")
+            raise TemplateSyntaxError("%r doesn't allow other block tags inside it" % bits[0])
         while parser.tokens:
             token = parser.next_token()
             if token.token_type in (TokenType.VAR, TokenType.TEXT):
                 plural.append(token)
             else:
                 break
-    if token.contents.strip() != 'endblocktrans':
-        raise TemplateSyntaxError("'blocktrans' doesn't allow other block tags (seen %r) inside it" % token.contents)
+    end_tag_name = 'end%s' % bits[0]
+    if token.contents.strip() != end_tag_name:
+        raise TemplateSyntaxError("%r doesn't allow other block tags (seen %r) inside it" % (bits[0], token.contents))
 
     return BlockTranslateNode(extra_context, singular, plural, countervar,
                               counter, message_context, trimmed=trimmed,
-                              asvar=asvar)
+                              asvar=asvar, tag_name=bits[0])
 
 
 @register.tag
