@@ -1,16 +1,11 @@
-# Avoid importing `importlib` from this package.
-from __future__ import absolute_import
-
 import datetime
 import decimal
 import unicodedata
 from importlib import import_module
 
 from django.conf import settings
-from django.utils import dateformat, datetime_safe, numberformat, six
-from django.utils.encoding import force_str
+from django.utils import dateformat, datetime_safe, numberformat
 from django.utils.functional import lazy
-from django.utils.safestring import mark_safe
 from django.utils.translation import (
     check_for_language, get_language, to_locale,
 )
@@ -22,14 +17,14 @@ _format_cache = {}
 _format_modules_cache = {}
 
 ISO_INPUT_FORMATS = {
-    'DATE_INPUT_FORMATS': ('%Y-%m-%d',),
-    'TIME_INPUT_FORMATS': ('%H:%M:%S', '%H:%M:%S.%f', '%H:%M'),
-    'DATETIME_INPUT_FORMATS': (
+    'DATE_INPUT_FORMATS': ['%Y-%m-%d'],
+    'TIME_INPUT_FORMATS': ['%H:%M:%S', '%H:%M:%S.%f', '%H:%M'],
+    'DATETIME_INPUT_FORMATS': [
         '%Y-%m-%d %H:%M:%S',
         '%Y-%m-%d %H:%M:%S.%f',
         '%Y-%m-%d %H:%M',
         '%Y-%m-%d'
-    ),
+    ],
 }
 
 
@@ -63,9 +58,7 @@ def reset_format_cache():
 
 
 def iter_format_modules(lang, format_module_path=None):
-    """
-    Does the heavy lifting of finding format modules.
-    """
+    """Find format modules."""
     if not check_for_language(lang):
         return
 
@@ -74,7 +67,7 @@ def iter_format_modules(lang, format_module_path=None):
 
     format_locations = []
     if format_module_path:
-        if isinstance(format_module_path, six.string_types):
+        if isinstance(format_module_path, str):
             format_module_path = [format_module_path]
         for path in format_module_path:
             format_locations.append(path + '.%s')
@@ -92,12 +85,12 @@ def iter_format_modules(lang, format_module_path=None):
 
 
 def get_format_modules(lang=None, reverse=False):
-    """
-    Returns a list of the format modules found
-    """
+    """Return a list of the format modules found."""
     if lang is None:
         lang = get_language()
-    modules = _format_modules_cache.setdefault(lang, list(iter_format_modules(lang, settings.FORMAT_MODULE_PATH)))
+    if lang not in _format_modules_cache:
+        _format_modules_cache[lang] = list(iter_format_modules(lang, settings.FORMAT_MODULE_PATH))
+    modules = _format_modules_cache[lang]
     if reverse:
         return list(reversed(modules))
     return modules
@@ -105,50 +98,53 @@ def get_format_modules(lang=None, reverse=False):
 
 def get_format(format_type, lang=None, use_l10n=None):
     """
-    For a specific format type, returns the format for the current
-    language (locale), defaults to the format in the settings.
-    format_type is the name of the format, e.g. 'DATE_FORMAT'
+    For a specific format type, return the format for the current
+    language (locale). Default to the format in the settings.
+    format_type is the name of the format, e.g. 'DATE_FORMAT'.
 
-    If use_l10n is provided and is not None, that will force the value to
+    If use_l10n is provided and is not None, it forces the value to
     be localized (or not), overriding the value of settings.USE_L10N.
     """
-    format_type = force_str(format_type)
-    if format_type not in FORMAT_SETTINGS:
-        return format_type
-    if use_l10n or (use_l10n is None and settings.USE_L10N):
-        if lang is None:
-            lang = get_language()
-        cache_key = (format_type, lang)
-        try:
-            cached = _format_cache[cache_key]
-            if cached is not None:
-                return cached
-            else:
-                # Return the general setting by default
-                return getattr(settings, format_type)
-        except KeyError:
-            for module in get_format_modules(lang):
-                try:
-                    val = getattr(module, format_type)
-                    for iso_input in ISO_INPUT_FORMATS.get(format_type, ()):
-                        if iso_input not in val:
-                            if isinstance(val, tuple):
-                                val = list(val)
-                            val.append(iso_input)
-                    _format_cache[cache_key] = val
-                    return val
-                except AttributeError:
-                    pass
-            _format_cache[cache_key] = None
-    return getattr(settings, format_type)
+    use_l10n = use_l10n or (use_l10n is None and settings.USE_L10N)
+    if use_l10n and lang is None:
+        lang = get_language()
+    cache_key = (format_type, lang)
+    try:
+        return _format_cache[cache_key]
+    except KeyError:
+        pass
 
-get_format_lazy = lazy(get_format, six.text_type, list, tuple)
+    # The requested format_type has not been cached yet. Try to find it in any
+    # of the format_modules for the given lang if l10n is enabled. If it's not
+    # there or if l10n is disabled, fall back to the project settings.
+    val = None
+    if use_l10n:
+        for module in get_format_modules(lang):
+            val = getattr(module, format_type, None)
+            if val is not None:
+                break
+    if val is None:
+        if format_type not in FORMAT_SETTINGS:
+            return format_type
+        val = getattr(settings, format_type)
+    elif format_type in ISO_INPUT_FORMATS:
+        # If a list of input formats from one of the format_modules was
+        # retrieved, make sure the ISO_INPUT_FORMATS are in this list.
+        val = list(val)
+        for iso_input in ISO_INPUT_FORMATS.get(format_type, ()):
+            if iso_input not in val:
+                val.append(iso_input)
+    _format_cache[cache_key] = val
+    return val
+
+
+get_format_lazy = lazy(get_format, str, list, tuple)
 
 
 def date_format(value, format=None, use_l10n=None):
     """
-    Formats a datetime.date or datetime.datetime object using a
-    localizable format
+    Format a datetime.date or datetime.datetime object using a
+    localizable format.
 
     If use_l10n is provided and is not None, that will force the value to
     be localized (or not), overriding the value of settings.USE_L10N.
@@ -158,9 +154,9 @@ def date_format(value, format=None, use_l10n=None):
 
 def time_format(value, format=None, use_l10n=None):
     """
-    Formats a datetime.time object using a localizable format
+    Format a datetime.time object using a localizable format.
 
-    If use_l10n is provided and is not None, that will force the value to
+    If use_l10n is provided and is not None, it forces the value to
     be localized (or not), overriding the value of settings.USE_L10N.
     """
     return dateformat.time_format(value, get_format(format or 'TIME_FORMAT', use_l10n=use_l10n))
@@ -168,9 +164,9 @@ def time_format(value, format=None, use_l10n=None):
 
 def number_format(value, decimal_pos=None, use_l10n=None, force_grouping=False):
     """
-    Formats a numeric value using localization settings
+    Format a numeric value using localization settings.
 
-    If use_l10n is provided and is not None, that will force the value to
+    If use_l10n is provided and is not None, it forces the value to
     be localized (or not), overriding the value of settings.USE_L10N.
     """
     if use_l10n or (use_l10n is None and settings.USE_L10N):
@@ -183,21 +179,26 @@ def number_format(value, decimal_pos=None, use_l10n=None, force_grouping=False):
         decimal_pos,
         get_format('NUMBER_GROUPING', lang, use_l10n=use_l10n),
         get_format('THOUSAND_SEPARATOR', lang, use_l10n=use_l10n),
-        force_grouping=force_grouping
+        force_grouping=force_grouping,
+        use_l10n=use_l10n,
     )
 
 
 def localize(value, use_l10n=None):
     """
-    Checks if value is a localizable type (date, number...) and returns it
+    Check if value is a localizable type (date, number...) and return it
     formatted as a string using current locale format.
 
-    If use_l10n is provided and is not None, that will force the value to
+    If use_l10n is provided and is not None, it forces the value to
     be localized (or not), overriding the value of settings.USE_L10N.
     """
-    if isinstance(value, bool):
-        return mark_safe(six.text_type(value))
-    elif isinstance(value, (decimal.Decimal, float) + six.integer_types):
+    if isinstance(value, str):  # Handle strings first for performance reasons.
+        return value
+    elif isinstance(value, bool):  # Make sure booleans don't get treated as numbers
+        return str(value)
+    elif isinstance(value, (decimal.Decimal, float, int)):
+        if use_l10n is False:
+            return str(value)
         return number_format(value, use_l10n=use_l10n)
     elif isinstance(value, datetime.datetime):
         return date_format(value, 'DATETIME_FORMAT', use_l10n=use_l10n)
@@ -205,37 +206,40 @@ def localize(value, use_l10n=None):
         return date_format(value, use_l10n=use_l10n)
     elif isinstance(value, datetime.time):
         return time_format(value, 'TIME_FORMAT', use_l10n=use_l10n)
-    else:
-        return value
+    return value
 
 
 def localize_input(value, default=None):
     """
-    Checks if an input value is a localizable type and returns it
+    Check if an input value is a localizable type and return it
     formatted with the appropriate formatting string of the current locale.
     """
-    if isinstance(value, (decimal.Decimal, float) + six.integer_types):
+    if isinstance(value, str):  # Handle strings first for performance reasons.
+        return value
+    elif isinstance(value, bool):  # Don't treat booleans as numbers.
+        return str(value)
+    elif isinstance(value, (decimal.Decimal, float, int)):
         return number_format(value)
     elif isinstance(value, datetime.datetime):
         value = datetime_safe.new_datetime(value)
-        format = force_str(default or get_format('DATETIME_INPUT_FORMATS')[0])
+        format = default or get_format('DATETIME_INPUT_FORMATS')[0]
         return value.strftime(format)
     elif isinstance(value, datetime.date):
         value = datetime_safe.new_date(value)
-        format = force_str(default or get_format('DATE_INPUT_FORMATS')[0])
+        format = default or get_format('DATE_INPUT_FORMATS')[0]
         return value.strftime(format)
     elif isinstance(value, datetime.time):
-        format = force_str(default or get_format('TIME_INPUT_FORMATS')[0])
+        format = default or get_format('TIME_INPUT_FORMATS')[0]
         return value.strftime(format)
     return value
 
 
 def sanitize_separators(value):
     """
-    Sanitizes a value according to the current decimal and
+    Sanitize a value according to the current decimal and
     thousand separator setting. Used with form field input.
     """
-    if settings.USE_L10N and isinstance(value, six.string_types):
+    if isinstance(value, str):
         parts = []
         decimal_separator = get_format('DECIMAL_SEPARATOR')
         if decimal_separator in value:
