@@ -3,7 +3,6 @@ from django.contrib.contenttypes.fields import (
 )
 from django.contrib.contenttypes.models import ContentType
 from django.db import models
-from django.db.models.deletion import ProtectedError
 
 __all__ = ('Link', 'Place', 'Restaurant', 'Person', 'Address',
            'CharLink', 'TextLink', 'OddRelation1', 'OddRelation2',
@@ -15,9 +14,6 @@ class Link(models.Model):
     object_id = models.PositiveIntegerField()
     content_object = GenericForeignKey()
 
-    def __str__(self):
-        return "Link to %s id=%s" % (self.content_type, self.object_id)
-
 
 class LinkProxy(Link):
     class Meta:
@@ -29,18 +25,13 @@ class Place(models.Model):
     links = GenericRelation(Link, related_query_name='places')
     link_proxy = GenericRelation(LinkProxy)
 
-    def __str__(self):
-        return "Place: %s" % self.name
-
 
 class Restaurant(Place):
-    def __str__(self):
-        return "Restaurant: %s" % self.name
+    pass
 
 
 class Cafe(Restaurant):
-    def __str__(self):
-        return "Cafe: %s" % self.name
+    pass
 
 
 class Address(models.Model):
@@ -52,18 +43,12 @@ class Address(models.Model):
     object_id = models.PositiveIntegerField()
     content_object = GenericForeignKey()
 
-    def __str__(self):
-        return '%s %s, %s %s' % (self.street, self.city, self.state, self.zipcode)
-
 
 class Person(models.Model):
     account = models.IntegerField(primary_key=True)
     name = models.CharField(max_length=128)
     addresses = GenericRelation(Address)
 
-    def __str__(self):
-        return self.name
-
 
 class CharLink(models.Model):
     content_type = models.ForeignKey(ContentType, models.CASCADE)
@@ -108,9 +93,6 @@ class Company(models.Model):
     name = models.CharField(max_length=100)
     links = GenericRelation(Link)
 
-    def __str__(self):
-        return "Company: %s" % self.name
-
 
 # For testing #13085 fix, we also use Note model defined above
 class Developer(models.Model):
@@ -121,9 +103,6 @@ class Team(models.Model):
     name = models.CharField(max_length=15)
     members = models.ManyToManyField(Developer)
 
-    def __str__(self):
-        return "%s team" % self.name
-
     def __len__(self):
         return self.members.count()
 
@@ -144,7 +123,7 @@ class Tag(models.Model):
 
 
 class Board(models.Model):
-    name = models.CharField(primary_key=True, max_length=15)
+    name = models.CharField(primary_key=True, max_length=25)
 
 
 class SpecialGenericRelation(GenericRelation):
@@ -214,7 +193,7 @@ class Related(models.Model):
 
 
 def prevent_deletes(sender, instance, **kwargs):
-    raise ProtectedError("Not allowed to delete.", [instance])
+    raise models.ProtectedError("Not allowed to delete.", [instance])
 
 
 models.signals.pre_delete.connect(prevent_deletes, sender=Node)
