@@ -1,26 +1,19 @@
 """
 Tests for geography support in PostGIS
 """
-from __future__ import unicode_literals
-
 import os
-from unittest import skipUnless
 
 from django.contrib.gis.db import models
 from django.contrib.gis.db.models.functions import Area, Distance
-from django.contrib.gis.gdal import HAS_GDAL
 from django.contrib.gis.measure import D
-from django.db import connection
+from django.db import NotSupportedError, connection
 from django.db.models.functions import Cast
-from django.test import TestCase, ignore_warnings, skipUnlessDBFeature
-from django.utils._os import upath
-from django.utils.deprecation import RemovedInDjango20Warning
+from django.test import TestCase, skipIfDBFeature, skipUnlessDBFeature
 
-from ..utils import oracle, postgis
+from ..utils import FuncTestMixin
 from .models import City, County, Zipcode
 
 
-@skipUnlessDBFeature("gis_enabled")
 class GeographyTest(TestCase):
     fixtures = ['initial']
 
@@ -30,7 +23,7 @@ class GeographyTest(TestCase):
 
     @skipUnlessDBFeature("supports_distances_lookups", "supports_distance_geodetic")
     def test02_distance_lookup(self):
-        "Testing GeoQuerySet distance lookup support on non-point geography fields."
+        "Testing distance lookup support on non-point geography fields."
         z = Zipcode.objects.get(code='77002')
         cities1 = list(City.objects
                        .filter(point__distance_lte=(z.poly, D(mi=500)))
@@ -43,17 +36,10 @@ class GeographyTest(TestCase):
         for cities in [cities1, cities2]:
             self.assertEqual(['Dallas', 'Houston', 'Oklahoma City'], cities)
 
-    @skipUnlessDBFeature("has_distance_method", "supports_distance_geodetic")
-    @ignore_warnings(category=RemovedInDjango20Warning)
-    def test03_distance_method(self):
-        "Testing GeoQuerySet.distance() support on non-point geography fields."
-        # `GeoQuerySet.distance` is not allowed geometry fields.
-        htown = City.objects.get(name='Houston')
-        Zipcode.objects.distance(htown.point)
-
-    @skipUnless(postgis, "This is a PostGIS-specific test")
     def test04_invalid_operators_functions(self):
         "Ensuring exceptions are raised for operators & functions invalid on geography fields."
+        if not connection.ops.postgis:
+            self.skipTest('This is a PostGIS-specific test.')
         # Only a subset of the geometry functions & operator are available
         # to PostGIS geography types.  For more information, visit:
         # http://postgis.refractions.net/documentation/manual-1.5/ch08.html#PostGIS_GeographyFunctions
@@ -70,7 +56,6 @@ class GeographyTest(TestCase):
         with self.assertRaises(ValueError):
             City.objects.get(point__exact=htown.point)
 
-    @skipUnless(HAS_GDAL, "GDAL is required.")
     def test05_geography_layermapping(self):
         "Testing LayerMapping support on models with geography fields."
         # There is a similar test in `layermap` that uses the same data set,
@@ -78,13 +63,13 @@ class GeographyTest(TestCase):
         from django.contrib.gis.utils import LayerMapping
 
         # Getting the shapefile and mapping dictionary.
-        shp_path = os.path.realpath(os.path.join(os.path.dirname(upath(__file__)), '..', 'data'))
+        shp_path = os.path.realpath(os.path.join(os.path.dirname(__file__), '..', 'data'))
         co_shp = os.path.join(shp_path, 'counties', 'counties.shp')
-        co_mapping = {'name': 'Name',
-                      'state': 'State',
-                      'mpoly': 'MULTIPOLYGON',
-                      }
-
+        co_mapping = {
+            'name': 'Name',
+            'state': 'State',
+            'mpoly': 'MULTIPOLYGON',
+        }
         # Reference county names, number of polygons, and state names.
         names = ['Bexar', 'Galveston', 'Harris', 'Honolulu', 'Pueblo']
         num_polys = [1, 2, 1, 19, 1]  # Number of polygons for each.
@@ -99,21 +84,8 @@ class GeographyTest(TestCase):
             self.assertEqual(name, c.name)
             self.assertEqual(state, c.state)
 
-    @skipUnlessDBFeature("has_area_method", "supports_distance_geodetic")
-    @ignore_warnings(category=RemovedInDjango20Warning)
-    def test06_geography_area(self):
-        "Testing that Area calculations work on geography columns."
-        # SELECT ST_Area(poly) FROM geogapp_zipcode WHERE code='77002';
-        z = Zipcode.objects.area().get(code='77002')
-        # Round to the nearest thousand as possible values (depending on
-        # the database and geolib) include 5439084, 5439100, 5439101.
-        rounded_value = z.area.sq_m
-        rounded_value -= z.area.sq_m % 1000
-        self.assertEqual(rounded_value, 5439000)
 
-
-@skipUnlessDBFeature("gis_enabled")
-class GeographyFunctionTests(TestCase):
+class GeographyFunctionTests(FuncTestMixin, TestCase):
     fixtures = ['initial']
 
     @skipUnlessDBFeature("supports_extent_aggr")
@@ -122,7 +94,7 @@ class GeographyFunctionTests(TestCase):
         Cast a geography to a geometry field for an aggregate function that
         expects a geometry input.
         """
-        if not connection.ops.geography:
+        if not connection.features.supports_geography:
             self.skipTest("This test needs geography support")
         expected = (-96.8016128540039, 29.7633724212646, -95.3631439208984, 32.782058715820)
         res = City.objects.filter(
@@ -136,19 +108,34 @@ class GeographyFunctionTests(TestCase):
         """
         Testing Distance() support on non-point geography fields.
         """
-        if oracle:
+        if connection.ops.oracle:
             ref_dists = [0, 4899.68, 8081.30, 9115.15]
+        elif connection.ops.spatialite:
+            # SpatiaLite returns non-zero distance for polygons and points
+            # covered by that polygon.
+            ref_dists = [326.61, 4899.68, 8081.30, 9115.15]
         else:
             ref_dists = [0, 4891.20, 8071.64, 9123.95]
         htown = City.objects.get(name='Houston')
-        qs = Zipcode.objects.annotate(distance=Distance('poly', htown.point))
+        qs = Zipcode.objects.annotate(
+            distance=Distance('poly', htown.point),
+            distance2=Distance(htown.point, 'poly'),
+        )
         for z, ref in zip(qs, ref_dists):
             self.assertAlmostEqual(z.distance.m, ref, 2)
-        # Distance function in combination with a lookup.
-        hzip = Zipcode.objects.get(code='77002')
-        self.assertEqual(qs.get(distance__lte=0), hzip)
 
-    @skipUnlessDBFeature("has_Area_function", "supports_distance_geodetic")
+        if connection.ops.postgis:
+            # PostGIS casts geography to geometry when distance2 is calculated.
+            ref_dists = [0, 4899.68, 8081.30, 9115.15]
+        for z, ref in zip(qs, ref_dists):
+            self.assertAlmostEqual(z.distance2.m, ref, 2)
+
+        if not connection.ops.spatialite:
+            # Distance function combined with a lookup.
+            hzip = Zipcode.objects.get(code='77002')
+            self.assertEqual(qs.get(distance__lte=0), hzip)
+
+    @skipUnlessDBFeature("has_Area_function", "supports_area_geodetic")
     def test_geography_area(self):
         """
         Testing that Area calculations work on geography columns.
@@ -160,3 +147,9 @@ class GeographyFunctionTests(TestCase):
         rounded_value = z.area.sq_m
         rounded_value -= z.area.sq_m % 1000
         self.assertEqual(rounded_value, 5439000)
+
+    @skipUnlessDBFeature("has_Area_function")
+    @skipIfDBFeature("supports_area_geodetic")
+    def test_geodetic_area_raises_if_not_supported(self):
+        with self.assertRaisesMessage(NotSupportedError, 'Area on geodetic coordinate systems not supported.'):
+            Zipcode.objects.annotate(area=Area('poly')).get(code='77002')
