@@ -66,28 +66,156 @@ class HumanizeTests(SimpleTestCase):
 
     def test_intcomma(self):
         test_list = (
-            100, 1000, 10123, 10311, 1000000, 1234567.25, '100', '1000',
-            '10123', '10311', '1000000', '1234567.1234567',
-            Decimal('1234567.1234567'), None,
+            100,
+            -100,
+            1000,
+            -1000,
+            10123,
+            -10123,
+            10311,
+            -10311,
+            1000000,
+            -1000000,
+            1234567.25,
+            -1234567.25,
+            "100",
+            "-100",
+            "100.1",
+            "-100.1",
+            "100.13",
+            "-100.13",
+            "1000",
+            "-1000",
+            "10123",
+            "-10123",
+            "10311",
+            "-10311",
+            "100000.13",
+            "-100000.13",
+            "1000000",
+            "-1000000",
+            "1234567.1234567",
+            "-1234567.1234567",
+            Decimal("1234567.1234567"),
+            Decimal("-1234567.1234567"),
+            None,
+            "１２３４５６７",
+            "-１２３４５６７",
+            "１２３４５６７.１２",
+            "-１２３４５６７.１２",
+            "the quick brown fox jumped over the lazy dog",
         )
         result_list = (
-            '100', '1,000', '10,123', '10,311', '1,000,000', '1,234,567.25',
-            '100', '1,000', '10,123', '10,311', '1,000,000', '1,234,567.1234567',
-            '1,234,567.1234567', None,
+            "100",
+            "-100",
+            "1,000",
+            "-1,000",
+            "10,123",
+            "-10,123",
+            "10,311",
+            "-10,311",
+            "1,000,000",
+            "-1,000,000",
+            "1,234,567.25",
+            "-1,234,567.25",
+            "100",
+            "-100",
+            "100.1",
+            "-100.1",
+            "100.13",
+            "-100.13",
+            "1,000",
+            "-1,000",
+            "10,123",
+            "-10,123",
+            "10,311",
+            "-10,311",
+            "100,000.13",
+            "-100,000.13",
+            "1,000,000",
+            "-1,000,000",
+            "1,234,567.1234567",
+            "-1,234,567.1234567",
+            "1,234,567.1234567",
+            "-1,234,567.1234567",
+            None,
+            "1,234,567",
+            "-1,234,567",
+            "１,２３４,５６７.１２",
+            "-１,２３４,５６７.１２",
+            "the quick brown fox jumped over the lazy dog",
         )
         with translation.override('en'):
             self.humanize_tester(test_list, result_list, 'intcomma')
 
     def test_l10n_intcomma(self):
         test_list = (
-            100, 1000, 10123, 10311, 1000000, 1234567.25, '100', '1000',
-            '10123', '10311', '1000000', '1234567.1234567',
-            Decimal('1234567.1234567'), None,
+            100,
+            -100,
+            1000,
+            -1000,
+            10123,
+            -10123,
+            10311,
+            -10311,
+            1000000,
+            -1000000,
+            1234567.25,
+            -1234567.25,
+            "100",
+            "-100",
+            "1000",
+            "-1000",
+            "10123",
+            "-10123",
+            "10311",
+            "-10311",
+            "1000000",
+            "-1000000",
+            "1234567.1234567",
+            "-1234567.1234567",
+            Decimal("1234567.1234567"),
+            -Decimal("1234567.1234567"),
+            None,
+            "１２３４５６７",
+            "-１２３４５６７",
+            "１２３４５６７.１２",
+            "-１２３４５６７.１２",
+            "the quick brown fox jumped over the lazy dog",
         )
         result_list = (
-            '100', '1,000', '10,123', '10,311', '1,000,000', '1,234,567.25',
-            '100', '1,000', '10,123', '10,311', '1,000,000', '1,234,567.1234567',
-            '1,234,567.1234567', None,
+            "100",
+            "-100",
+            "1,000",
+            "-1,000",
+            "10,123",
+            "-10,123",
+            "10,311",
+            "-10,311",
+            "1,000,000",
+            "-1,000,000",
+            "1,234,567.25",
+            "-1,234,567.25",
+            "100",
+            "-100",
+            "1,000",
+            "-1,000",
+            "10,123",
+            "-10,123",
+            "10,311",
+            "-10,311",
+            "1,000,000",
+            "-1,000,000",
+            "1,234,567.1234567",
+            "-1,234,567.1234567",
+            "1,234,567.1234567",
+            "-1,234,567.1234567",
+            None,
+            "1,234,567",
+            "-1,234,567",
+            "１,２３４,５６７.１２",
+            "-１,２３４,５６７.１２",
+            "the quick brown fox jumped over the lazy dog",
         )
         with self.settings(USE_L10N=True, USE_THOUSAND_SEPARATOR=False):
             with translation.override('en'):
