@@ -11,11 +11,10 @@ class ManyToOneRecursiveTests(TestCase):
         cls.c = Category.objects.create(id=None, name='Child category', parent=cls.r)
 
     def test_m2o_recursive(self):
-        self.assertQuerysetEqual(self.r.child_set.all(),
-                                 ['<Category: Child category>'])
+        self.assertSequenceEqual(self.r.child_set.all(), [self.c])
         self.assertEqual(self.r.child_set.get(name__startswith='Child').id, self.c.id)
         self.assertIsNone(self.r.parent)
-        self.assertQuerysetEqual(self.c.child_set.all(), [])
+        self.assertSequenceEqual(self.c.child_set.all(), [])
         self.assertEqual(self.c.parent.id, self.r.id)
 
 
@@ -30,9 +29,7 @@ class MultipleManyToOneRecursiveTests(TestCase):
     def test_m2o_recursive2(self):
         self.assertEqual(self.kid.mother.id, self.mom.id)
         self.assertEqual(self.kid.father.id, self.dad.id)
-        self.assertQuerysetEqual(self.dad.fathers_child_set.all(),
-                                 ['<Person: John Smith Junior>'])
-        self.assertQuerysetEqual(self.mom.mothers_child_set.all(),
-                                 ['<Person: John Smith Junior>'])
-        self.assertQuerysetEqual(self.kid.mothers_child_set.all(), [])
-        self.assertQuerysetEqual(self.kid.fathers_child_set.all(), [])
+        self.assertSequenceEqual(self.dad.fathers_child_set.all(), [self.kid])
+        self.assertSequenceEqual(self.mom.mothers_child_set.all(), [self.kid])
+        self.assertSequenceEqual(self.kid.mothers_child_set.all(), [])
+        self.assertSequenceEqual(self.kid.fathers_child_set.all(), [])
