@@ -6,13 +6,10 @@ from django.http import HttpRequest, HttpResponse
 
 class MiddlewareTests(unittest.TestCase):
 
-    def setUp(self):
-        self.middleware = MessageMiddleware()
-
     def test_response_without_messages(self):
         """
         MessageMiddleware is tolerant of messages not existing on request.
         """
         request = HttpRequest()
         response = HttpResponse()
-        self.middleware.process_response(request, response)
+        MessageMiddleware(lambda req: HttpResponse()).process_response(request, response)
